@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,14 +26,19 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	file.c,v 2.26 1994/03/21 05:45:27 jrs Exp
+ *	$Id: file.c,v 2.37 1996/05/21 03:20:56 jrs Exp $
  */
 
 #include "defs.h"
 
-#include <sys/dirent.h>
+#include <dirent.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+
+#ifdef SVR4
+#include <sys/cred.h>
+#endif /* SVR4 */
+
 #include <sys/vfs.h>
 
 #ifdef MAJOR_IN_SYSMACROS
@@ -142,7 +143,7 @@ struct tcb *tcp;
 		printflags(openmodes, tcp->u_arg[1] + 1);
 		if (tcp->u_arg[1] & O_CREAT) {
 			/* mode */
-			tprintf(", %#o", tcp->u_arg[2]);
+			tprintf(", %#lo", tcp->u_arg[2]);
 		}
 	}
 	return 0;
@@ -154,7 +155,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %#o", tcp->u_arg[1]);
+		tprintf(", %#lo", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -164,6 +165,12 @@ static struct xlat access_flags[] = {
 	{ R_OK,		"R_OK"		},
 	{ W_OK,		"W_OK"		},
 	{ X_OK,		"X_OK"		},
+#ifdef EFF_ONLY_OK
+	{ EFF_ONLY_OK,	"EFF_ONLY_OK"	},
+#endif
+#ifdef EX_OK
+	{ EX_OK,	"EX_OK"		},
+#endif
 	{ 0,		NULL		},
 };
 
@@ -184,7 +191,7 @@ sys_umask(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%#o", tcp->u_arg[0]);
+		tprintf("%#lo", tcp->u_arg[0]);
 	}
 	return RVAL_OCTAL;
 }
@@ -201,11 +208,11 @@ sys_lseek(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		if (tcp->u_arg[2] == SEEK_SET)
-			tprintf("%u, ", tcp->u_arg[1]);
+			tprintf("%lu, ", tcp->u_arg[1]);
 		else
-			tprintf("%d, ", tcp->u_arg[1]);
+			tprintf("%ld, ", tcp->u_arg[1]);
 		printxval(whence, tcp->u_arg[2], "L_???");
 	}
 	return RVAL_UDECIMAL;
@@ -217,7 +224,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %u", tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -227,7 +234,7 @@ sys_ftruncate(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, %u", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%ld, %lu", tcp->u_arg[0], tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -271,7 +278,7 @@ int mode;
 
 static char *
 sprinttime(t)
-unsigned long t;
+time_t t;
 {
 	struct tm *tmp;
 	static char buf[32];
@@ -368,7 +375,7 @@ sys_fstat(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp))
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	else {
 		printstat(tcp, tcp->u_arg[1]);
 	}
@@ -395,7 +402,7 @@ sys_xstat(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printpath(tcp, tcp->u_arg[1]);
 		tprintf(", ");
 	} else {
@@ -409,7 +416,7 @@ sys_fxstat(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp))
-		tprintf("%d, %d, ", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%ld, %ld, ", tcp->u_arg[0], tcp->u_arg[1]);
 	else {
 		printstat(tcp, tcp->u_arg[2]);
 	}
@@ -421,7 +428,7 @@ sys_lxstat(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printpath(tcp, tcp->u_arg[1]);
 		tprintf(", ");
 	} else {
@@ -437,7 +444,7 @@ struct tcb *tcp;
 	int mode = tcp->u_arg[2];
 
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printpath(tcp, tcp->u_arg[1]);
 		tprintf(", %s", sprintmode(mode));
 		switch (mode & S_IFMT) {
@@ -485,8 +492,10 @@ int magic;
 	char *s;
 
 	s = xlookup(fsmagic, magic);
-	if (s)
-		return s;
+	if (s) {
+		sprintf(buf, "\"%s\"", s);
+		return buf;
+	}
 #endif /* LINUX */
 	sprintf(buf, "%#x", magic);
 	return buf;
@@ -495,18 +504,26 @@ int magic;
 static void
 printstatfs(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	struct statfs statbuf;
 
 	if (syserror(tcp) || !verbose(tcp)) {
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 		return;
 	}
 	if (umove(tcp, addr, &statbuf) < 0) {
 		tprintf("{...}");
 		return;
 	}
+#ifdef ALPHA
+
+	tprintf("{f_type=%s, f_fsize=%u, f_fbsize=%u, f_blocks=%u, f_bfree=%u, ",
+		sprintfstype(statbuf.f_type),
+		statbuf.f_fsize,statbuf.f_bsize, statbuf.f_blocks, statbuf.f_bfree);
+	tprintf("f_bavail=%u, f_files=%u, f_ffree=%u, f_namelen=%u}",
+		statbuf.f_bavail,statbuf.f_files, statbuf.f_ffree, statbuf.f_namelen);
+#else /* !ALPHA */
 	tprintf("{f_type=%s, f_bsize=%lu, f_blocks=%lu, f_bfree=%lu, ",
 		sprintfstype(statbuf.f_type),
 		statbuf.f_bsize, statbuf.f_blocks, statbuf.f_bfree);
@@ -515,6 +532,7 @@ int addr;
 #ifdef LINUX
 	tprintf(", f_namelen=%lu}", statbuf.f_namelen);
 #endif /* LINUX */
+#endif /* !ALPHA */
 	tprintf("}");
 }
 
@@ -536,12 +554,45 @@ sys_fstatfs(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 	} else {
 		printstatfs(tcp, tcp->u_arg[1]);
 	}
 	return 0;
 }
+
+#ifdef LINUX
+#ifdef ALPHA
+
+int
+osf_statfs(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+	} else {
+		printstatfs(tcp, tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[2]);
+	}
+	return 0;
+}
+
+int
+osf_fstatfs(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		tprintf("%lu, ", tcp->u_arg[0]);
+	} else {
+		printstatfs(tcp, tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[2]);
+	}
+	return 0;
+}
+#endif /* ALPHA */
+#endif /* LINUX */
+
 #endif /* !SVR4 */
 
 #ifdef SUNOS4
@@ -553,12 +604,13 @@ struct tcb *tcp;
 	struct ustat statbuf;
 
 	if (entering(tcp)) {
-		tprintf("makedev(%u, %u), ",
-				major(tcp->u_arg[0]), minor(tcp->u_arg[0]));
+		tprintf("makedev(%lu, %lu), ",
+				(long) major(tcp->u_arg[0]),
+				(long) minor(tcp->u_arg[0]));
 	}
 	else {
 		if (syserror(tcp) || !verbose(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else if (umove(tcp, tcp->u_arg[1], &statbuf) < 0)
 			tprintf("{...}");
 		else {
@@ -594,7 +646,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %#o", tcp->u_arg[1]);
+		tprintf(", %#lo", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -614,7 +666,7 @@ sys_fchdir(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d", tcp->u_arg[0]);
+		tprintf("%ld", tcp->u_arg[0]);
 	}
 	return 0;
 }
@@ -634,7 +686,7 @@ sys_fchroot(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d", tcp->u_arg[0]);
+		tprintf("%ld", tcp->u_arg[0]);
 	}
 	return 0;
 }
@@ -682,10 +734,10 @@ struct tcb *tcp;
 		tprintf(", ");
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printpathn(tcp, tcp->u_arg[1], tcp->u_rval);
-		tprintf(", %u", tcp->u_arg[2]);
+		tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
@@ -708,7 +760,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %u, %u", tcp->u_arg[1], tcp->u_arg[2]);
+		tprintf(", %lu, %lu", tcp->u_arg[1], tcp->u_arg[2]);
 	}
 	return 0;
 }
@@ -718,7 +770,7 @@ sys_fchown(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, %u, %u",
+		tprintf("%ld, %lu, %lu",
 			tcp->u_arg[0], tcp->u_arg[1], tcp->u_arg[2]);
 	}
 	return 0;
@@ -730,7 +782,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %#o", tcp->u_arg[1]);
+		tprintf(", %#lo", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -740,7 +792,7 @@ sys_fchmod(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, %#o", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%ld, %#lo", tcp->u_arg[0], tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -769,7 +821,7 @@ struct tcb *tcp;
 		if (!tcp->u_arg[1])
 			tprintf("NULL");
 		else if (!verbose(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else if (umoven(tcp, tcp->u_arg[1], sizeof ut,
 		    (char *) ut) < 0)
 			tprintf("[?, ?]");
@@ -809,7 +861,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %#o", tcp->u_arg[1]);
+		tprintf(", %#lo", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -819,7 +871,7 @@ sys_fsync(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d", tcp->u_arg[0]);
+		tprintf("%ld", tcp->u_arg[0]);
 	}
 	return 0;
 }
@@ -829,12 +881,12 @@ struct tcb *tcp;
 static void
 printdir(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	struct dirent d;
 
 	if (!verbose(tcp)) {
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 		return;
 	}
 	if (umove(tcp, addr, &d) < 0) {
@@ -851,7 +903,7 @@ int addr;
 	tprintf("d_reclen=%u, ", d.d_reclen);
 #endif /* !LINUX */
 	tprintf("d_name=");
-	printpathn(tcp, (int) ((struct dirent *) addr)->d_name, d.d_reclen);
+	printpathn(tcp, (long) ((struct dirent *) addr)->d_name, d.d_reclen);
 	tprintf("}");
 }
 
@@ -860,23 +912,21 @@ sys_readdir(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp) || tcp->u_rval == 0 || !verbose(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printdir(tcp, tcp->u_arg[1]);
 		/* Not much point in printing this out, it is always 1. */
 		if (tcp->u_arg[2] != 1)
-			tprintf(", %u", tcp->u_arg[2]);
+			tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
 
 #endif /* LINUX */
 
-#ifdef HAVE_GETDENTS
-
 int
 sys_getdents(tcp)
 struct tcb *tcp;
@@ -885,11 +935,11 @@ struct tcb *tcp;
 	char *buf;
 
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 		return 0;
 	}
 	if (syserror(tcp) || !verbose(tcp)) {
-		tprintf("%#x, %u", tcp->u_arg[1], tcp->u_arg[2]);
+		tprintf("%#lx, %lu", tcp->u_arg[1], tcp->u_arg[2]);
 		return 0;
 	}
 	len = tcp->u_rval;
@@ -898,14 +948,22 @@ struct tcb *tcp;
 		return 0;
 	}
 	if (umoven(tcp, tcp->u_arg[1], len, buf) < 0) {
-		tprintf("{...}, %u", tcp->u_arg[2]);
+		tprintf("{...}, %lu", tcp->u_arg[2]);
 		free(buf);
 		return 0;
 	}
 	if (!abbrev(tcp))
 		tprintf("{");
 	for (i = 0; i < len;) {
-		struct dirent *d = (struct dirent *)&buf[i];
+		struct dirent *d = (struct dirent *) &buf[i];
+#ifdef LINUX
+		if (!abbrev(tcp)) {
+			tprintf("%s{d_ino=%lu, d_off=%lu, ",
+				i ? " " : "", d->d_ino, d->d_off);
+			tprintf("d_reclen=%u, d_name=\"%s\"}",
+				d->d_reclen, d->d_name);
+		}
+#endif /* LINUX */
 #ifdef SVR4
 		if (!abbrev(tcp)) {
 			tprintf("%s{d_ino=%lu, d_off=%lu, ",
@@ -929,14 +987,12 @@ struct tcb *tcp;
 	if (!abbrev(tcp))
 		tprintf("}");
 	else
-		tprintf("[total %u dents]", dents);
-	tprintf(", %u", tcp->u_arg[2]);
+		tprintf("/* %u entries */", dents);
+	tprintf(", %lu", tcp->u_arg[2]);
 	free(buf);
 	return 0;
 }
 
-#endif /* HAVE_GETDENTS */
-
 #ifdef HAVE_SYS_ASYNCH_H
 
 int
@@ -946,17 +1002,17 @@ struct tcb *tcp;
 	struct aio_result_t res;
 
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
-		tprintf(", %u, %u, ", tcp->u_arg[2], tcp->u_arg[3]);
+		tprintf(", %lu, %lu, ", tcp->u_arg[2], tcp->u_arg[3]);
 		printxval(whence, tcp->u_arg[4], "L_???");
 		if (syserror(tcp) || tcp->u_arg[5] == 0
 		    || umove(tcp, tcp->u_arg[5], &res) < 0)
-			tprintf(", %#x", tcp->u_arg[5]);
+			tprintf(", %#lx", tcp->u_arg[5]);
 		else
 			tprintf(", {aio_return %d aio_errno %d}",
 				res.aio_return, res.aio_errno);
@@ -971,9 +1027,9 @@ struct tcb *tcp;
 	struct aio_result_t res;
 
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
-		tprintf(", %u, %u, ", tcp->u_arg[2], tcp->u_arg[3]);
+		tprintf(", %lu, %lu, ", tcp->u_arg[2], tcp->u_arg[3]);
 		printxval(whence, tcp->u_arg[4], "L_???");
 	}
 	else {
@@ -981,7 +1037,7 @@ struct tcb *tcp;
 			tprintf(", NULL");
 		else if (syserror(tcp)
 		    || umove(tcp, tcp->u_arg[5], &res) < 0)
-			tprintf(", %#x", tcp->u_arg[5]);
+			tprintf(", %#lx", tcp->u_arg[5]);
 		else
 			tprintf(", {aio_return %d aio_errno %d}",
 				res.aio_return, res.aio_errno);
@@ -1009,7 +1065,7 @@ struct tcb *tcp;
 			tprintf("NULL");
 		else if (syserror(tcp)
 		    || umove(tcp, tcp->u_arg[0], &res) < 0)
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		else
 			tprintf("{aio_return %d aio_errno %d}",
 				res.aio_return, res.aio_errno);
