@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	io.c,v 2.24 1994/06/29 04:43:21 jrs Exp
+ *	$Id: io.c,v 2.29 1996/05/19 06:40:36 jrs Exp $
  */
 
 #include "defs.h"
@@ -43,13 +39,13 @@ sys_read(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printstr(tcp, tcp->u_arg[1], tcp->u_rval);
-		tprintf(", %u", tcp->u_arg[2]);
+		tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
@@ -59,15 +55,13 @@ sys_write(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
-		tprintf(", %u", tcp->u_arg[2]);
+		tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
 
-#ifndef LINUX
-
 int
 sys_readv(tcp)
 struct tcb *tcp;
@@ -76,10 +70,10 @@ struct tcb *tcp;
 	int i, len;
 
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp)) {
-			tprintf("%#x, %u",
+			tprintf("%#lx, %lu",
 					tcp->u_arg[1], tcp->u_arg[2]);
 			return 0;
 		}
@@ -90,21 +84,21 @@ struct tcb *tcp;
 		}
 		if (umoven(tcp, tcp->u_arg[1],
 				len * sizeof *iov, (char *) iov) < 0) {
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		} else {
 			tprintf("[");
 			for (i = 0; i < len; i++) {
 				if (i)
 					tprintf(", ");
 				tprintf("{");
-				printstr(tcp, (int) iov[i].iov_base,
+				printstr(tcp, (long) iov[i].iov_base,
 					iov[i].iov_len);
 				tprintf(", %u}", iov[i].iov_len);
 			}
 			tprintf("]");
 		}
 		free((char *) iov);
-		tprintf(", %u", tcp->u_arg[2]);
+		tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
@@ -117,7 +111,7 @@ struct tcb *tcp;
 	int i, len;
 
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		len = tcp->u_arg[2];
 		iov = (struct iovec *) malloc(len * sizeof *iov);
 		if (iov == NULL) {
@@ -126,27 +120,25 @@ struct tcb *tcp;
 		}
 		if (umoven(tcp, tcp->u_arg[1],
 				len * sizeof *iov, (char *) iov) < 0) {
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		} else {
 			tprintf("[");
 			for (i = 0; i < len; i++) {
 				if (i)
 					tprintf(", ");
 				tprintf("{");
-				printstr(tcp, (int) iov[i].iov_base,
+				printstr(tcp, (long) iov[i].iov_base,
 					iov[i].iov_len);
 				tprintf(", %u}", iov[i].iov_len);
 			}
 			tprintf("]");
 		}
 		free((char *) iov);
-		tprintf(", %u", tcp->u_arg[2]);
+		tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
 
-#endif /* !LINUX */
-
 #ifdef SVR4
 
 int
@@ -154,13 +146,13 @@ sys_pread(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printstr(tcp, tcp->u_arg[1], tcp->u_rval);
-		tprintf(", %u, %u", tcp->u_arg[2], tcp->u_arg[3]);
+		tprintf(", %lu, %lu", tcp->u_arg[2], tcp->u_arg[3]);
 	}
 	return 0;
 }
@@ -170,9 +162,9 @@ sys_pwrite(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
-		tprintf(", %u, %u", tcp->u_arg[2], tcp->u_arg[3]);
+		tprintf(", %lu, %lu", tcp->u_arg[2], tcp->u_arg[3]);
 	}
 	return 0;
 }
@@ -186,17 +178,17 @@ struct tcb *tcp;
 	char *symbol;
 
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		symbol = ioctl_lookup(tcp->u_arg[1]);
 		if (symbol)
 			tprintf("%s", symbol);
 		else
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		ioctl_decode(tcp, tcp->u_arg[1], tcp->u_arg[2]);
 	}
 	else {
 		if (ioctl_decode(tcp, tcp->u_arg[1], tcp->u_arg[2]) == 0)
-			tprintf(", %#x", tcp->u_arg[2]);
+			tprintf(", %#lx", tcp->u_arg[2]);
 	}
 	return 0;
 }
