@@ -25,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: syscallent.h,v 1.1 1996/05/17 02:38:46 jrs Exp $
+ *	$Id: syscallent.h,v 1.2 1996/10/28 03:31:10 jrs Exp $
  */
 
 	{ 5,	0,	printargs,		"SYS_0"		}, /* 0 */
@@ -369,7 +369,7 @@
 	{ 5,	0,	sys_afs_syscall,	"afs_syscall"	}, /* 338 */
 	{ 1,	0,	sys_uname,		"uname"		}, /* 339 */
 	{ 2,	0,	sys_nanosleep,		"nanosleep"	}, /* 340 */
-	{ 5,	0,	printargs,		"SYS_341"	}, /* 341 */
+	{ 5,	0,	sys_mremap,		"mremap"	}, /* 341 */
 	{ 5,	0,	printargs,		"SYS_342"	}, /* 342 */
 	{ 5,	0,	printargs,		"SYS_343"	}, /* 343 */
 	{ 5,	0,	printargs,		"SYS_344"	}, /* 344 */
