@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -11,11 +11,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -29,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	dummy.h,v 2.19 1994/03/12 23:12:13 jrs Exp
+ *	$Id: dummy.h,v 2.25 1996/05/21 03:21:05 jrs Exp $
  */
 
 /* still unfinished */
@@ -38,22 +34,46 @@
 #define	sys_syslog		printargs
 #define	sys_iopl		printargs
 #define	sys_vm86		printargs
-#define sys_modify_ldt		printargs
-#define sys_init_module		printargs
-#define sys_get_kernel_syms	printargs
-#define sys_quotactl		printargs
+#define	sys_modify_ldt		printargs
+#define	sys_init_module		printargs
+#define	sys_get_kernel_syms	printargs
+#define	sys_quotactl		printargs
+#define	sys_bdflush		printargs
+#define	sys_sysfs		printargs
+#define	sys_afs_syscall		printargs
+#define	sys_llseek		printargs
+
+#define sys_fdatasync		printargs
+#define sys_sysctl		printargs
+#define sys_mlock		printargs
+#define sys_munlock		printargs
+#define sys_mlockall		printargs
+#define sys_munlockall		printargs
+#define sys_sched_setparam	printargs
+#define sys_sched_getparam	printargs
+#define sys_sched_setscheduler	printargs
+#define sys_sched_getscheduler	printargs
+#define sys_sched_yield		printargs
+#define sys_sched_get_priority_max printargs
+#define sys_sched_get_priority_min printargs
+#define sys_sched_rr_get_interval printargs
+#define sys_nanosleep		printargs
+#define sys_mremap		printargs
 
 /* like another call */
-#define sys_uselib		sys_chdir
+#define	sys_uselib		sys_chdir
 #define	sys_umount		sys_chdir
 #define	sys_swapon		sys_chdir
 #define	sys_swapoff		sys_chdir
 #define	sys_clone		sys_fork
-#define sys_create_module	sys_sethostname
-#define sys_delete_module	sys_chdir
+#define	sys_create_module	sys_sethostname
+#define	sys_delete_module	sys_chdir
+#define	sys_fchdir		sys_close
+#define	sys_setfsuid		sys_setuid
+#define	sys_setfsgid		sys_setgid
 
 /* printargs does the right thing */
-#define sys_setup		printargs
+#define	sys_setup		printargs
 #define	sys_getpid		printargs
 #define	sys_getuid		printargs
 #define	sys_pause		printargs
@@ -66,28 +86,11 @@
 #define	sys_setsid		printargs
 #define	sys_vhangup		printargs
 #define	sys_idle		printargs
-#define sys_getpgid		printargs
-#define sys_fchdir		printargs
-#define	sys_bdflush		printargs
-
-#define	sys_sysfs		printargs
-#define	sys_personality		printargs
-#define	sys_afs_syscall		printargs
-#define	sys_setfsuid		printargs
-#define	sys_setfsgid		printargs
-#define	sys__llseek		printargs
-#define	sys__newselect		printargs
-#define	sys_flock		printargs
-#define	sys_syscall_flock	printargs
-#define	sys_msync		printargs
-#define	sys_readv		printargs
-#define	sys_syscall_readv	printargs
-#define	sys_writev		printargs
-#define	sys_syscall_writev	printargs
+#define	sys_getpgid		printargs
 
 /* subcall entry points */
-#define sys_socketcall		printargs
-#define sys_ipc			printargs
+#define	sys_socketcall		printargs
+#define	sys_ipc			printargs
 
 /* unimplemented */
 #define	sys_stty		printargs
@@ -110,30 +113,25 @@
 #define	sys_olduname		printargs
 #define	sys_oldolduname		printargs
 
-/* new in linux 1.[1-9] since strace 3.0 */
-#define sys_sysfs              printargs
-#define sys_afs_syscall                printargs
-#define sys__llseek            printargs
-#define sys__newselect         printargs
-#define sys_flock              printargs
-#define sys_msync              printargs
-#define sys_readv              printargs
-#define sys_writev             printargs
-#define sys_getsid             printargs
-#define sys_mlock              printargs
-#define sys_munlock            printargs
-#define sys_mlockall           printargs
-#define sys_munlockall         printargs
+/* no library support */
+#ifndef HAVE_SENDMSG
+#define sys_sendmsg		printargs
+#define sys_recvmsg		printargs
+#endif
 
 #if DONE
+#define sys_oldselect		printargs
+#define	sys_msync		printargs
+#define	sys_flock		printargs
+#define	sys_getdents		printargs
 #define	sys_stime		printargs
 #define	sys_time		printargs
 #define	sys_times		printargs
 #define	sys_mount		printargs
 #define	sys_nice		printargs
-#define sys_mprotect		printargs
-#define sys_sigprocmask		printargs
-#define sys_adjtimex		printargs
+#define	sys_mprotect		printargs
+#define	sys_sigprocmask		printargs
+#define	sys_adjtimex		printargs
 #define	sys_sysinfo		printargs
 #define	sys_ipc			printargs
 #define	sys_setdomainname	printargs
@@ -212,16 +210,5 @@
 #define	sys_stat		printargs
 #define	sys_lstat		printargs
 #define	sys_fstat		printargs
+#define	sys_personality		printargs
 #endif
-
-#define	sys_semget		printargs
-#define	sys_semctl		printargs
-#define	sys_msgsnd		printargs
-#define	sys_msgrcv		printargs
-#define	sys_msgget		printargs
-#define	sys_msgctl		printargs
-#define	sys_shmat		printargs
-#define	sys_shmdt		printargs
-#define	sys_shmget		printargs
-#define	sys_shmctl		printargs
-
