@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -11,11 +11,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -29,7 +25,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	syscall.h,v 2.18 1994/03/12 23:12:15 jrs Exp
+ *	$Id: syscall.h,v 2.26 1996/05/21 03:21:06 jrs Exp $
  */
 
 #include "dummy.h"
@@ -56,7 +52,7 @@ int sys_setreuid(), sys_setregid(), sys_sigsuspend();
 int sys_sigpending(), sys_sethostname(), sys_setrlimit();
 int sys_getrlimit(), sys_getrusage(), sys_gettimeofday();
 int sys_settimeofday(), sys_getgroups(), sys_setgroups();
-int sys_select(), sys_symlink(), sys_oldlstat(), sys_readlink();
+int sys_oldselect(), sys_symlink(), sys_oldlstat(), sys_readlink();
 int sys_uselib(), sys_swapon(), sys_reboot(), sys_readdir();
 int sys_mmap(), sys_munmap(), sys_truncate(), sys_ftruncate();
 int sys_fchmod(), sys_fchown(), sys_getpriority();
@@ -69,14 +65,18 @@ int sys_fsync(), sys_clone(), sys_setdomainname(), sys_sysinfo();
 int sys_modify_ldt(), sys_adjtimex(), sys_mprotect();
 int sys_sigprocmask(), sys_create_module(), sys_init_module();
 int sys_delete_module(), sys_get_kernel_syms(), sys_quotactl();
-int sys_getpgid(), sys_fchdir(), sys_bdflush(), sys_personality();
-
-int sys_sysfs(), sys_personality(), sys_afs_syscall(), sys_setfsuid();
-int sys_setfsgid(), sys__llseek(), sys_getdents(), sys__newselect();
-int sys_flock(), sys_syscall_flock(), sys_msync(), sys_readv();
-int sys_syscall_readv(), sys_writev(), sys_syscall_writev();
-int sys_getsid();
-
+int sys_getpgid(), sys_fchdir(), sys_bdflush();
+int sys_sysfs(), sys_personality(), sys_afs_syscall();
+int sys_setfsuid(), sys_setfsgid(), sys_llseek();
+int sys_getdents(), sys_flock(), sys_msync();
+int sys_readv(), sys_writev(), sys_select();
+int sys_getsid(), sys_fdatasync(), sys_sysctl();
+int sys_mlock(), sys_munlock(), sys_mlockall(), sys_munlockall();
+int sys_sched_setparam(), sys_sched_getparam();
+int sys_sched_setscheduler(), sys_sched_getscheduler(), sys_sched_yield();
+int sys_sched_get_priority_max(), sys_sched_get_priority_min();
+int sys_sched_rr_get_interval(), sys_nanosleep(), sys_mremap();
+int sys_sendmsg(), sys_recvmsg();
 
 /* sys_socketcall subcalls */
 
@@ -85,7 +85,8 @@ int sys_accept(), sys_getsockname(), sys_getpeername(), sys_socketpair();
 int sys_send(), sys_recv(), sys_sendto(), sys_recvfrom();
 int sys_shutdown(), sys_setsockopt(), sys_getsockopt();
 
-#define SYS_socket_subcall	180	/* WTA: was 150? */
+#ifndef ALPHA
+#define SYS_socket_subcall	180
 #define SYS_socket		(SYS_socket_subcall + 1)
 #define SYS_bind		(SYS_socket_subcall + 2)
 #define SYS_connect		(SYS_socket_subcall + 3)
@@ -103,6 +104,7 @@ int sys_shutdown(), sys_setsockopt(), sys_getsockopt();
 #define SYS_getsockopt		(SYS_socket_subcall + 15)
 
 #define SYS_socket_nsubcalls	16
+#endif /* !ALPHA */
 
 /* sys_ipc subcalls */
 
@@ -110,7 +112,8 @@ int sys_semget(), sys_semctl();
 int sys_msgsnd(), sys_msgrcv(), sys_msgget(), sys_msgctl();
 int sys_shmat(), sys_shmdt(), sys_shmget(), sys_shmctl();
 
-#define SYS_ipc_subcall		200 /* WTA: was 170? */
+#ifndef ALPHA
+#define SYS_ipc_subcall		200
 #define SYS_semop		(SYS_ipc_subcall + 1)
 #define SYS_semget		(SYS_ipc_subcall + 2)
 #define SYS_semctl		(SYS_ipc_subcall + 3)
@@ -124,4 +127,9 @@ int sys_shmat(), sys_shmdt(), sys_shmget(), sys_shmctl();
 #define SYS_shmctl		(SYS_ipc_subcall + 24)
 
 #define SYS_ipc_nsubcalls	25
+#endif /* ALPHA */
 
+#ifdef ALPHA
+int osf_statfs(), osf_fstatfs(), sys_getpagesize(), sys_madvise();
+int sys_setpgrp(), sys_gethostname(), sys_getdtablesize(), sys_utimes();
+#endif
