@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	net.c,v 2.21 1994/03/21 05:45:31 jrs Exp
+ *	$Id: net.c,v 2.29 1996/05/21 03:20:57 jrs Exp $
  */
 
 #include "defs.h"
@@ -40,42 +36,25 @@
 #include <sys/un.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#ifdef LINUX
-#include <linux/ipx.h>
-#endif
 
 #ifndef PF_UNSPEC
 #define PF_UNSPEC AF_UNSPEC
 #endif
 
+#ifdef LINUX
+/* Under Linux these are enums so we can't test for them with ifdef. */
+#define IPPROTO_EGP IPPROTO_EGP
+#define IPPROTO_PUP IPPROTO_PUP
+#define IPPROTO_IDP IPPROTO_IDP
+#define IPPROTO_IGMP IPPROTO_IGMP
+#define IPPROTO_RAW IPPROTO_RAW
+#define IPPROTO_MAX IPPROTO_MAX
+#endif
+
 static struct xlat domains[] = {
 	{ PF_UNSPEC,	"PF_UNSPEC"	},
 	{ PF_UNIX,	"PF_UNIX"	},
 	{ PF_INET,	"PF_INET"	},
-#ifdef PF_AX25
-       { PF_AX25,      "PF_AX25"       },
-#endif
-#ifdef PF_IPX
-       { PF_IPX,       "PF_IPX"        },
-#endif
-#ifdef PF_APPLETALK
-       { PF_APPLETALK, "PF_APPLETALK"  },
-#endif
-#ifdef PF_NETROM
-       { PF_NETROM,    "PF_NETROM"     },
-#endif
-#ifdef PF_BRIDGE
-       { PF_BRIDGE,    "PF_BRIDGE"     },
-#endif
-#ifdef PF_AAL5
-       { PF_AAL5,      "PF_AAL5"       },
-#endif
-#ifdef PF_X25
-       { PF_X25,       "PF_X25"        },
-#endif
-#ifdef PF_INET6
-       { PF_INET6,     "PF_INET6"      },
-#endif
 #ifdef PF_ISO
 	{ PF_ISO,	"PF_ISO"	},
 #endif
@@ -97,9 +76,8 @@ static struct xlat socktypes[] = {
 	{ SOCK_RDM,	"SOCK_RDM"	},
 #endif
 #ifdef SOCK_PACKET
-       { SOCK_PACKET,  "SOCK_PACKET"   },
+	{ SOCK_PACKET,	"SOCK_PACKET"	},
 #endif
-
 	{ 0,		NULL		},
 };
 static struct xlat protocols[] = {
@@ -134,10 +112,6 @@ static struct xlat protocols[] = {
 #ifdef IPPROTO_MAX
 	{ IPPROTO_MAX,	"IPPROTO_MAX"	},
 #endif
-#ifdef IPPROTO_IPIP
-       { IPPROTO_IPIP, "IPPROTO_IPIP"  },
-#endif
-
 	{ 0,		NULL		},
 };
 static struct xlat msg_flags[] = {
@@ -151,16 +125,6 @@ static struct xlat msg_flags[] = {
 	{ 0,		NULL		},
 };
 
-static struct xlat ipxtypes[] = {
-       /* Sorry, they are private??? */
-       { 0,            "IPX_TYPE_UNKNOWN" },
-       { 1,            "IPX_TYPE_RIP"  },
-       { 4,            "IPX_TYPE_SAP"  },
-       { 5,            "IPX_TYPE_SPX"  },
-       { 0x11,         "IPX_TYPE_NCP"  },
-       { 0x14,         "IPX_TYPE_PPROP" },
-};
-
 static struct xlat sockoptions[] = {
 #ifdef SO_DEBUG
 	{ SO_DEBUG,	"SO_DEBUG"	},
@@ -219,64 +183,24 @@ static struct xlat sockoptions[] = {
 #ifdef SO_RCVTIMEO
 	{ SO_RCVTIMEO,	"SO_RCVTIMEO"	},
 #endif
-#ifdef SO_BSDCOMPAT
-       { SO_BSDCOMPAT, "SO_BSDCOMPAT"  },
-#endif
-
 	{ 0,		NULL		},
 };
 
-#ifdef SOL_IP
-static struct xlat sockipoptions[] = {
-       { IP_TOS,       "IP_TOS"        },
-       { IP_TTL,       "IP_TTL"        },
-#if defined(IP_HDRINCL)
-       { IP_HDRINCL,   "IP_HDRINCL"    },
-#endif
-#if defined(IP_OPTIONS)
-       { IP_OPTIONS,   "IP_OPTIONS"    },
-#endif
-       { IP_MULTICAST_IF,      "IP_MULTICAST_IF"       },
-       { IP_MULTICAST_TTL,     "IP_MULTICAST_TTL"      },
-       { IP_MULTICAST_LOOP,    "IP_MULTICAST_LOOP"     },
-       { IP_ADD_MEMBERSHIP,    "IP_ADD_MEMBERSHIP"     },
-       { IP_DROP_MEMBERSHIP,   "IP_DROP_MEMBERSHIP"    },
-       { 0,            NULL            },
-};
-#endif
-#ifdef SOL_IPX
-static struct xlat sockipxoptions[] = {
-       { IPX_TYPE,     "IPX_TYPE"      },
-       { 0,            NULL            },
-};
-#endif
-
-#ifdef SOL_TCP
-static struct xlat socktcpoptions[] = {
-       { TCP_NODELAY,  "TCP_NODELAY"   },
-       { TCP_MAXSEG,   "TCP_MAXSEG"    },
-       { 0,            NULL            },
-};
-#endif
-
-
 void
 printsock(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	struct sockaddr sa;
 	struct sockaddr_in *sin = (struct sockaddr_in *) &sa;
 	struct sockaddr_un sau;
-	struct sockaddr_ipx sipx;
-	
 
 	if (addr == 0) {
 		tprintf("NULL");
 		return;
 	}
 	if (!verbose(tcp)) {
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 		return;
 	}
 	if (umove(tcp, addr, &sa) < 0) {
@@ -296,54 +220,39 @@ int addr;
 		tprintf("sin_port=htons(%u), sin_addr=inet_addr(\"%s\")}",
 			ntohs(sin->sin_port), inet_ntoa(sin->sin_addr));
 		break;
-       case AF_IPX:
-               if (umove(tcp, addr, &sipx) < 0)
-                       tprintf("{sipx_family=AF_IPX, ...}");
-               else {
-                       int i;
-                       tprintf("{sipx_family=AF_IPX, ");
-                       tprintf("{sipx_port=htons(%u), ",
-                               ntohs(sipx.sipx_port));
-                       /* Yes, I know, this does not look too
-                          strace-ish, but otherwise the IPX
-                          addresses just look monstrous...
-                          Anyways, feel free if you don't like
-                          this way.. :) */
-                       tprintf("%08lx:", ntohl(sipx.sipx_network));
-                       for (i = 0; i<IPX_NODE_LEN; i++)
-                               tprintf("%02x", sipx.sipx_node[i]);
-                       tprintf("/[%02x]", sipx.sipx_type);
-                       tprintf("}");
-               }
-               break;
-       /* AF_AX25 AF_IPX AF_APPLETALK AF_NETROM AF_BRIDGE AF_AAL5
-          AF_X25 AF_INET6 Still need to be done */
 	default:
 		tprintf("{sa_family=%u, sa_data=", sa.sa_family);
-		printstr(tcp, (int) &((struct sockaddr *) addr)->sa_data,
+		printstr(tcp, (long) &((struct sockaddr *) addr)->sa_data,
 			sizeof sa.sa_data);
 		tprintf("}");
 		break;
 	}
 }
 
-#ifdef SUNOS4
+#if HAVE_SENDMSG
 
 static void
 printmsghdr(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	struct msghdr msg;
 
 	umove(tcp, addr, &msg);
 	tprintf("{msg_name=%s, msg_namelen=%u, msg_iov=%#lx, msg_iovlen=%u, ",
-		msg.msg_name, msg.msg_namelen,
+		(char *) msg.msg_name, msg.msg_namelen,
 		(unsigned long) msg.msg_iov, msg.msg_iovlen);
+#ifdef HAVE_MSG_CONTROL
+	tprintf("msg_control=%#lx, msg_controllen=%u, msg_flags=%#x}",
+		(unsigned long) msg.msg_control, msg.msg_controllen,
+		msg.msg_flags);
+#else /* !HAVE_MSG_CONTROL */
 	tprintf("msg_accrights=%#lx, msg_accrightslen=%u}",
 		(unsigned long) msg.msg_accrights, msg.msg_accrightslen);
+#endif /* !HAVE_MSG_CONTROL */
 }
-#endif
+
+#endif /* HAVE_SENDMSG */
 
 int
 sys_socket(tcp)
@@ -354,20 +263,10 @@ struct tcb *tcp;
 		tprintf(", ");
 		printxval(socktypes, tcp->u_arg[1], "SOCK_???");
 		tprintf(", ");
-		switch (tcp->u_arg[0]) {
-		case PF_INET:
+		if (tcp->u_arg[0] == PF_INET)
 			printxval(protocols, tcp->u_arg[2], "IPPROTO_???");
-			break;
-               case PF_IPX:
-                       /* BTW, I don't believe this.. */
-                       tprintf("[");
-                       printxval(domains, tcp->u_arg[2], "PF_???");
-                       tprintf("]");
-                       break;
-               default:
-                       tprintf("%u", tcp->u_arg[2]);
-                       break;
-		}
+		else
+			tprintf("%lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
@@ -377,9 +276,9 @@ sys_bind(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printsock(tcp, tcp->u_arg[1]);
-		tprintf(", %u", tcp->u_arg[2]);
+		tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
@@ -396,7 +295,7 @@ sys_listen(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, %u", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%ld, %lu", tcp->u_arg[0], tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -406,17 +305,17 @@ sys_accept(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else if (!tcp->u_arg[2])
-		tprintf("%#x, NULL", tcp->u_arg[1]);
+		tprintf("%#lx, NULL", tcp->u_arg[1]);
 	else {
 		if (tcp->u_arg[1] == 0 || syserror(tcp)) {
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		} else {
 			printsock(tcp, tcp->u_arg[1]);
 		}
 		tprintf(", ");
-		printnum(tcp, tcp->u_arg[2], "%u");
+		printnum(tcp, tcp->u_arg[2], "%lu");
 	}
 	return 0;
 }
@@ -426,9 +325,9 @@ sys_send(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
-		tprintf(", %u, ", tcp->u_arg[2]);
+		tprintf(", %lu, ", tcp->u_arg[2]);
 		/* flags */
 		if (printflags(msg_flags, tcp->u_arg[3]) == 0)
 			tprintf("0");
@@ -441,9 +340,9 @@ sys_sendto(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printstr(tcp, tcp->u_arg[1], tcp->u_arg[2]);
-		tprintf(", %u, ", tcp->u_arg[2]);
+		tprintf(", %lu, ", tcp->u_arg[2]);
 		/* flags */
 		if (printflags(msg_flags, tcp->u_arg[3]) == 0)
 			tprintf("0");
@@ -451,19 +350,19 @@ struct tcb *tcp;
 		tprintf(", ");
 		printsock(tcp, tcp->u_arg[4]);
 		/* to length */
-		tprintf(", %u", tcp->u_arg[5]);
+		tprintf(", %lu", tcp->u_arg[5]);
 	}
 	return 0;
 }
 
-#ifdef SUNOS4
+#ifdef HAVE_SENDMSG
 
 int
 sys_sendmsg(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 		printmsghdr(tcp, tcp->u_arg[1]);
 		/* flags */
 		tprintf(", ");
@@ -473,21 +372,21 @@ struct tcb *tcp;
 	return 0;
 }
 
-#endif /* SUNOS4 */
+#endif /* HAVE_SENDMSG */
 
 int
 sys_recv(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printstr(tcp, tcp->u_arg[1], tcp->u_rval);
 
-		tprintf(", %u, ", tcp->u_arg[2]);
+		tprintf(", %lu, ", tcp->u_arg[2]);
 		if (printflags(msg_flags, tcp->u_arg[3]) == 0)
 			tprintf("0");
 	}
@@ -499,12 +398,12 @@ sys_recvfrom(tcp)
 struct tcb *tcp;
 {
 	int fromlen;
-	
+
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp)) {
-			tprintf("%#x, %u, %u, %#x, %#x",
+			tprintf("%#lx, %lu, %lu, %#lx, %#lx",
 				tcp->u_arg[1], tcp->u_arg[2], tcp->u_arg[3],
 				tcp->u_arg[4], tcp->u_arg[5]);
 			return 0;
@@ -512,7 +411,7 @@ struct tcb *tcp;
 		/* buf */
 		printstr(tcp, tcp->u_arg[1], tcp->u_rval);
 		/* len */
-		tprintf(", %u, ", tcp->u_arg[2]);
+		tprintf(", %lu, ", tcp->u_arg[2]);
 		/* flags */
 		if (printflags(msg_flags, tcp->u_arg[3]) == 0)
 			tprintf("0");
@@ -521,11 +420,11 @@ struct tcb *tcp;
 			if (tcp->u_arg[4] == 0)
 				tprintf(", NULL");
 			else
-				tprintf(", %#x", tcp->u_arg[4]);
+				tprintf(", %#lx", tcp->u_arg[4]);
 			if (tcp->u_arg[5] == 0)
 				tprintf(", NULL");
 			else
-				tprintf(", %#x", tcp->u_arg[5]);
+				tprintf(", %#lx", tcp->u_arg[5]);
 			return 0;
 		}
 		if (umove(tcp, tcp->u_arg[5], &fromlen) < 0) {
@@ -540,17 +439,17 @@ struct tcb *tcp;
 	return 0;
 }
 
-#ifdef SUNOS4
+#ifdef HAVE_SENDMSG
 
 int
 sys_recvmsg(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else {
 		if (syserror(tcp) || !verbose(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printmsghdr(tcp, tcp->u_arg[1]);
 		/* flags */
@@ -561,14 +460,14 @@ struct tcb *tcp;
 	return 0;
 }
 
-#endif /* SUNOS4 */
+#endif /* HAVE_SENDMSG */
 
 int
 sys_shutdown(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%d, %d", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%ld, %ld", tcp->u_arg[0], tcp->u_arg[1]);
 		switch (tcp->u_arg[1]) {
 		case 0:
 			tprintf("%s", " /* receive */");
@@ -608,7 +507,7 @@ struct tcb *tcp;
 
 	if (exiting(tcp)) {
 		if (syserror(tcp)) {
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 			return 0;
 		}
 		if (umoven(tcp, tcp->u_arg[0], sizeof fds, (char *) fds) < 0)
@@ -620,12 +519,12 @@ struct tcb *tcp;
 
 #ifdef SUNOS4
 	if (exiting(tcp))
-		tprintf("[%u, %u]", tcp->u_rval, getrval2(tcp));
+		tprintf("[%lu, %lu]", tcp->u_rval, getrval2(tcp));
 #endif /* SUNOS4 */
 
 #ifdef SVR4
 	if (exiting(tcp))
-		tprintf("[%u, %u]", tcp->u_rval, getrval2(tcp));
+		tprintf("[%lu, %lu]", tcp->u_rval, getrval2(tcp));
 #endif /* SVR4 */
 
 	return 0;
@@ -644,23 +543,14 @@ struct tcb *tcp;
 		tprintf(", ");
 		printxval(socktypes, tcp->u_arg[1], "SOCK_???");
 		tprintf(", ");
-		switch (tcp->u_arg[0]) {
-		case PF_INET:
+		if (tcp->u_arg[0] == PF_INET) {
 			printxval(protocols, tcp->u_arg[2], "IPPROTO_???");
-			break;
-               case PF_IPX:
-                       /* BTW, I don't believe this.. */
-                       tprintf("[");
-                       printxval(domains, tcp->u_arg[2], "PF_???");
-                       tprintf("]");
-                       break;
-		default:
-			tprintf(", %u", tcp->u_arg[2]);
-			break;
+		} else {
+			tprintf(", %lu", tcp->u_arg[2]);
 		}
 	} else {
 		if (syserror(tcp)) {
-			tprintf("%#x", tcp->u_arg[3]);
+			tprintf("%#lx", tcp->u_arg[3]);
 			return 0;
 		}
 #ifdef LINUX
@@ -670,10 +560,10 @@ struct tcb *tcp;
 			tprintf(", [%u, %u]", fds[0], fds[1]);
 #endif /* LINUX */
 #ifdef SUNOS4
-		tprintf(", [%u, %u]", tcp->u_rval, getrval2(tcp));
+		tprintf(", [%lu, %lu]", tcp->u_rval, getrval2(tcp));
 #endif /* SUNOS4 */
 #ifdef SVR4
-		tprintf(", [%u, %u]", tcp->u_rval, getrval2(tcp));
+		tprintf(", [%lu, %lu]", tcp->u_rval, getrval2(tcp));
 #endif /* SVR4 */
 	}
 	return 0;
@@ -684,50 +574,25 @@ sys_getsockopt(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
-		switch (tcp->u_arg[1]) {
-		case SOL_SOCKET:
+		tprintf("%ld, ", tcp->u_arg[0]);
+		if (tcp->u_arg[1] == SOL_SOCKET) {
 			tprintf("SOL_SOCKET, ");
 			printxval(sockoptions, tcp->u_arg[2], "SO_???");
 			tprintf(", ");
-			break;
-#ifdef SOL_IP
-               case SOL_IP:
-                       tprintf("SOL_IP, ");
-                       printxval(sockipoptions, tcp->u_arg[2], "IP_???");
-                       tprintf(", ");
-                       break;
-#endif
-#ifdef SOL_IPX
-               case SOL_IPX:
-                       tprintf("SOL_IPX, ");
-                       printxval(sockipxoptions, tcp->u_arg[2], "IPX_???");
-                       tprintf(", ");
-                       break;
-#endif
-#ifdef SOL_TCP
-               case SOL_TCP:
-                       tprintf("SOL_TCP, ");
-                       printxval(socktcpoptions, tcp->u_arg[2], "TCP_???");
-                       tprintf(", ");
-                       break;
-#endif
-               /* SOL_AX25 SOL_ATALK SOL_NETROM SOL_UDP XXX still need work */
-		default:
+		} else {
 			/* XXX - should know socket family here */
 			printxval(protocols, tcp->u_arg[1], "IPPROTO_???");
-			tprintf("%u, ", tcp->u_arg[2]);
-			break;
+			tprintf("%lu, ", tcp->u_arg[2]);
 		}
 	} else {
 		if (syserror(tcp)) {
-			tprintf("%#x, %#x",
+			tprintf("%#lx, %#lx",
 				tcp->u_arg[3], tcp->u_arg[4]);
 			return 0;
 		}
-		printnum(tcp, tcp->u_arg[3], "%d");
+		printnum(tcp, tcp->u_arg[3], "%ld");
 		tprintf(", ");
-		printnum(tcp, tcp->u_arg[4], "%d");
+		printnum(tcp, tcp->u_arg[4], "%ld");
 	}
 	return 0;
 }
@@ -737,42 +602,18 @@ sys_setsockopt(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
-		switch (tcp->u_arg[1]) {
-		case SOL_SOCKET:
+		tprintf("%ld, ", tcp->u_arg[0]);
+		if (tcp->u_arg[1] == SOL_SOCKET) {
 			tprintf("SOL_SOCKET, ");
 			printxval(sockoptions, tcp->u_arg[2], "SO_???");
 			tprintf(", ");
-			break;
-#ifdef SOL_IP
-               case SOL_IP:
-                       tprintf("SOL_IP, ");
-                       printxval(sockipoptions, tcp->u_arg[2], "IP_???");
-                       tprintf(", ");
-                       break;
-#endif
-#ifdef SOL_IPX
-               case SOL_IPX:
-                       tprintf("SOL_IPX, ");
-                       printxval(sockipxoptions, tcp->u_arg[2], "IPX_???");
-                       tprintf(", ");
-                       break;
-#endif
-#ifdef SOL_TCP
-               case SOL_TCP:
-                       tprintf("SOL_TCP, ");
-                       printxval(socktcpoptions, tcp->u_arg[2], "TCP_???");
-                       tprintf(", ");
-                       break;
-#endif
-		default:
+		} else {
 			/* XXX - should know socket family here */
 			printxval(protocols, tcp->u_arg[1], "IPPROTO_???");
-			tprintf("%u, ", tcp->u_arg[2]);
-			break;
+			tprintf("%lu, ", tcp->u_arg[2]);
 		}
-		printnum(tcp, tcp->u_arg[3], "%d");
-		tprintf(", %u", tcp->u_arg[4]);
+		printnum(tcp, tcp->u_arg[3], "%ld");
+		tprintf(", %lu", tcp->u_arg[4]);
 	}
 	return 0;
 }
