@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	process.c,v 2.25 1994/06/29 04:43:22 jrs Exp
+ *	$Id: process.c,v 2.36 1996/05/19 06:40:36 jrs Exp $
  */
 
 #include "defs.h"
@@ -42,15 +38,61 @@
 #include <sys/resource.h>
 #include <sys/utsname.h>
 #include <sys/user.h>
+#include <sys/syscall.h>
 #include <signal.h>
 #ifdef SUNOS4
 #include <machine/reg.h>
 #endif
 
+#ifdef HAVE_PRCTL
+#include <sys/prctl.h>
+#endif
+
 #ifndef WCOREDUMP
 #define WCOREDUMP(status) ((status) & 0200)
 #endif
 
+#ifdef HAVE_PRCTL
+static struct xlat prctl_options[] = {
+	{ PR_MAXPROCS,		"PR_MAXPROCS"		},
+	{ PR_ISBLOCKED,		"PR_ISBLOCKED"		},
+	{ PR_SETSTACKSIZE,	"PR_SETSTACKSIZE"	},
+	{ PR_GETSTACKSIZE,	"PR_GETSTACKSIZE"	},
+	{ PR_MAXPPROCS,		"PR_MAXPPROCS"		},
+	{ PR_UNBLKONEXEC,	"PR_UNBLKONEXEC"	},
+	{ PR_ATOMICSIM,		"PR_ATOMICSIM"		},
+	{ PR_SETEXITSIG,	"PR_SETEXITSIG"		},
+	{ PR_RESIDENT,		"PR_RESIDENT"		},
+	{ PR_ATTACHADDR,	"PR_ATTACHADDR"		},
+	{ PR_DETACHADDR,	"PR_DETACHADDR"		},
+	{ PR_TERMCHILD,		"PR_TERMCHILD"		},
+	{ PR_GETSHMASK,		"PR_GETSHMASK"		},
+	{ PR_GETNSHARE,		"PR_GETNSHARE"		},
+	{ 0,			NULL			},
+};
+
+int
+sys_prctl(tcp)
+struct tcb *tcp;
+{
+	int i;
+
+	if (entering(tcp)) {
+		printxval(prctl_options, tcp->u_arg[0], "PR_???");
+		switch (tcp->u_arg[0]) {
+		case PR_GETNSHARE:
+			break;
+		default:
+			for (i = 1; i < tcp->u_nargs; i++)
+				tprintf(", %#lx", tcp->u_arg[i]);
+			break;
+		}
+	}
+	return 0;
+}
+
+#endif /* HAVE_PRCTL */
+
 int
 sys_gethostid(tcp)
 struct tcb *tcp;
@@ -66,7 +108,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpathn(tcp, tcp->u_arg[0], tcp->u_arg[1]);
-		tprintf(", %u", tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -77,10 +119,10 @@ struct tcb *tcp;
 {
 	if (exiting(tcp)) {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		else
 			printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %u", tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -91,7 +133,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 		printpathn(tcp, tcp->u_arg[0], tcp->u_arg[1]);
-		tprintf(", %u", tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -104,10 +146,10 @@ struct tcb *tcp;
 {
 	if (exiting(tcp)) {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		else
 			printpath(tcp, tcp->u_arg[0]);
-		tprintf(", %u", tcp->u_arg[1]);
+		tprintf(", %lu", tcp->u_arg[1]);
 	}
 	return 0;
 }
@@ -122,11 +164,19 @@ struct tcb *tcp;
 		return -1;
 	}
 	/* special case: we stop tracing this process, finish line now */
-	tprintf("%d) ", tcp->u_arg[0]);
+	tprintf("%ld) ", tcp->u_arg[0]);
 	tabto(acolumn);
 	tprintf("= ?");
 	printtrailer(tcp);
-	tcp->flags |= TCB_EXITING;
+	return 0;
+}
+
+int
+internal_exit(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp))
+		tcp->flags |= TCB_EXITING;
 	return 0;
 }
 
@@ -136,13 +186,24 @@ int
 sys_fork(tcp)
 struct tcb *tcp;
 {
-	struct tcb *tcpchild;
-
 	if (exiting(tcp)) {
 		if (getrval2(tcp)) {
 			tcp->auxstr = "child process";
 			return RVAL_UDECIMAL | RVAL_STR;
 		}
+	}
+	return 0;
+}
+
+int
+internal_fork(tcp)
+struct tcb *tcp;
+{
+	struct tcb *tcpchild;
+
+	if (exiting(tcp)) {
+		if (getrval2(tcp))
+			return 0;
 		if (!followfork)
 			return 0;
 		if (nprocs == MAX_PROCS) {
@@ -165,15 +226,26 @@ struct tcb *tcp;
 
 #else /* !SVR4 */
 
-static int vforking;
-
 int
 sys_fork(tcp)
 struct tcb *tcp;
+{
+	if (exiting(tcp))
+		return RVAL_UDECIMAL;
+	return 0;
+}
+
+int
+internal_fork(tcp)
+struct tcb *tcp;
 {
 	struct tcb *tcpchild;
 	int pid;
+	int vforking = 0;
 
+#ifdef SYS_vfork
+	vforking = (tcp->scno == EXEC_vfork);
+#endif
 	if (entering(tcp)) {
 		if (!followfork || vforking)
 			return 0;
@@ -267,13 +339,9 @@ int
 sys_vfork(tcp)
 struct tcb *tcp;
 {
-	/* XXX - cannot use the same trick for vfork */
-	int res;
-
-	vforking = 1;
-	res = sys_fork(tcp);
-	vforking = 0;
-	return res;
+	if (exiting(tcp))
+		return RVAL_UDECIMAL;
+	return 0;
 }
 
 #endif /* SUNOS4 */
@@ -287,7 +355,7 @@ sys_getpid(tcp)
 struct tcb *tcp;
 {
 	if (exiting(tcp)) {
-		sprintf(idstr, "ppid %u", getrval2(tcp));
+		sprintf(idstr, "ppid %lu", getrval2(tcp));
 		tcp->auxstr = idstr;
 		return RVAL_STR;
 	}
@@ -299,7 +367,7 @@ sys_getuid(tcp)
 struct tcb *tcp;
 {
 	if (exiting(tcp)) {
-		sprintf(idstr, "euid %u", getrval2(tcp));
+		sprintf(idstr, "euid %lu", getrval2(tcp));
 		tcp->auxstr = idstr;
 		return RVAL_STR;
 	}
@@ -311,7 +379,7 @@ sys_getgid(tcp)
 struct tcb *tcp;
 {
 	if (exiting(tcp)) {
-		sprintf(idstr, "egid %u", getrval2(tcp));
+		sprintf(idstr, "egid %lu", getrval2(tcp));
 		tcp->auxstr = idstr;
 		return RVAL_STR;
 	}
@@ -388,7 +456,7 @@ struct tcb *tcp;
 			return -1;
 		}
 		if (!verbose(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else if (umoven(tcp, tcp->u_arg[1],
 		    len * sizeof(GETGROUPS_T), (char *) gidset) < 0)
 			tprintf("[?]");
@@ -428,7 +496,7 @@ struct tcb *tcp;
 		if (!tcp->u_arg[1])
 			tprintf("NULL");
 		else if (!verbose(tcp) || tcp->u_arg[0] == 0)
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else if (umoven(tcp, tcp->u_arg[1],
 		    len * sizeof(GETGROUPS_T), (char *) gidset) < 0)
 			tprintf("[?]");
@@ -450,7 +518,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 #ifndef SVR4
-		tprintf("%u, %u", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%lu, %lu", tcp->u_arg[0], tcp->u_arg[1]);
 #endif /* !SVR4 */
 	}
 	return 0;
@@ -462,7 +530,7 @@ struct tcb *tcp;
 {
 	if (entering(tcp)) {
 #ifndef SVR4
-		tprintf("%u", tcp->u_arg[0]);
+		tprintf("%lu", tcp->u_arg[0]);
 #endif /* !SVR4 */
 	}
 	return 0;
@@ -473,7 +541,7 @@ sys_getsid(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u", tcp->u_arg[0]);
+		tprintf("%lu", tcp->u_arg[0]);
 	}
 	return 0;
 }
@@ -490,7 +558,7 @@ sys_getpgid(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u", tcp->u_arg[0]);
+		tprintf("%lu", tcp->u_arg[0]);
 	}
 	return 0;
 }
@@ -500,29 +568,57 @@ sys_setpgid(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, %u", tcp->u_arg[0], tcp->u_arg[1]);
+		tprintf("%lu, %lu", tcp->u_arg[0], tcp->u_arg[1]);
 	}
 	return 0;
 }
 
+void
+fake_execve(tcp, program, argv, envp)
+struct tcb *tcp;
+char *program;
+char *argv[];
+char *envp[];
+{
+	int i;
+
+	if (!(qual_flags[SYS_execve] & QUAL_TRACE))
+		return;
+	printleader(tcp);
+	tprintf("execve(");
+	string_quote(program);
+	tprintf(", [");
+	for (i = 0; argv[i] != NULL; i++) {
+		if (i != 0)
+			tprintf(", ");
+		string_quote(argv[i]);
+	}
+	for (i = 0; envp[i] != NULL; i++)
+		;
+	tprintf("], [/* %d var%s */]) ", i, (i != 1) ? "s" : "");
+	tabto(acolumn);
+	tprintf("= 0");
+	printtrailer(tcp);
+}
+
 static void
 printargv(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
-	int cp;
+	char *cp;
 	char *sep;
 	int max = max_strlen / 2;
 
 	for (sep = ""; --max >= 0; sep = ", ") {
 		if (umove(tcp, addr, &cp) < 0) {
-			tprintf("%#x", addr);
+			tprintf("%#lx", addr);
 			return;
 		}
 		if (cp == 0)
 			break;
 		tprintf(sep);
-		printstr(tcp, cp, -1);
+		printstr(tcp, (long) cp, -1);
 		addr += sizeof(char *);
 	}
 	if (cp)
@@ -533,16 +629,12 @@ static void
 printargc(fmt, tcp, addr)
 char *fmt;
 struct tcb *tcp;
-int addr;
+long addr;
 {
-	int cp;
 	int count;
+	char *cp;
 
-	for (count = 0; ; count++) {
-		if (umove(tcp, addr, &cp) < 0)
-			break;
-		if (cp == 0)
-			break;
+	for (count = 0; umove(tcp, addr, &cp) >= 0 && cp != NULL; count++) {
 		addr += sizeof(char *);
 	}
 	tprintf(fmt, count, count == 1 ? "" : "s");
@@ -555,10 +647,10 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
 		if (!verbose(tcp))
-			tprintf(", %#x", tcp->u_arg[1]);
+			tprintf(", %#lx", tcp->u_arg[1]);
 #if 0
 		else if (abbrev(tcp))
-			printargc(", [%d arg%s]", tcp, tcp->u_arg[1]);
+			printargc(", [/* %d arg%s */]", tcp, tcp->u_arg[1]);
 #endif
 		else {
 			tprintf(", [");
@@ -566,10 +658,6 @@ struct tcb *tcp;
 			tprintf("]");
 		}
 	}
-#ifdef SUNOS4
-	else if (!syserror(tcp) && followfork)
-		fixvfork(tcp);
-#endif /* SUNOS4 */
 	return 0;
 }
 
@@ -580,10 +668,10 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		printpath(tcp, tcp->u_arg[0]);
 		if (!verbose(tcp))
-			tprintf(", %#x", tcp->u_arg[1]);
+			tprintf(", %#lx", tcp->u_arg[1]);
 #if 0
 		else if (abbrev(tcp))
-			printargc(", [%d arg%s]", tcp, tcp->u_arg[1]);
+			printargc(", [/* %d arg%s */]", tcp, tcp->u_arg[1]);
 #endif
 		else {
 			tprintf(", [");
@@ -591,17 +679,29 @@ struct tcb *tcp;
 			tprintf("]");
 		}
 		if (!verbose(tcp))
-			tprintf(", %#x", tcp->u_arg[2]);
+			tprintf(", %#lx", tcp->u_arg[2]);
 		else if (abbrev(tcp))
-			printargc(", [%d var%s]", tcp, tcp->u_arg[2]);
+			printargc(", [/* %d var%s */]", tcp, tcp->u_arg[2]);
 		else {
 			tprintf("[");
 			printargv(tcp, tcp->u_arg[2]);
 			tprintf("]");
 		}
 	}
+#ifdef LINUX
+#ifdef ALPHA
+	tcp->flags |= TCB_WAITEXECVE;
+#endif /* ALPHA */
+#endif /* LINUX */
+	return 0;
+}
+
+int
+internal_exec(tcp)
+struct tcb *tcp;
+{
 #ifdef SUNOS4
-	else if (!syserror(tcp) && followfork)
+	if (exiting(tcp) && !syserror(tcp) && followfork)
 		fixvfork(tcp);
 #endif /* SUNOS4 */
 	return 0;
@@ -647,14 +747,14 @@ int status;
 
 	/*
 	 * Here is a tricky presentation problem.  This solution
-	 * is still not entirely satisfactory because there
-	 * are no wait status constructors.
+	 * is still not entirely satisfactory but since there
+	 * are no wait status constructors it will have to do.
 	 */
 	if (WIFSTOPPED(status))
 		tprintf("[WIFSTOPPED(s) && WSTOPSIG(s) == %s]",
 			signalent[WSTOPSIG(status)]);
 	else if WIFSIGNALED(status)
-		tprintf("[WIFSTOPPED(s) && WTERMSIG(s) == %s%s]",
+		tprintf("[WIFSIGNALED(s) && WTERMSIG(s) == %s%s]",
 			signalent[WTERMSIG(status)],
 			WCOREDUMP(status) ? " && WCOREDUMP(s)" : "");
 	else if WIFEXITED(status) {
@@ -673,21 +773,16 @@ struct tcb *tcp;
 int n;
 {
 	int status;
-	int exited;
+	int exited = 0;
 
 	if (entering(tcp)) {
-		tprintf("%d, ", tcp->u_arg[0]);
-		if (tcp->nchildren > 0) {
-			/* There are traced children */
-			tcp->flags |= TCB_SUSPENDED;
-			tcp->waitpid = tcp->u_arg[0];
-		}
+		tprintf("%ld, ", tcp->u_arg[0]);
 	} else {
 		/* status */
 		if (!tcp->u_arg[1])
 			tprintf("NULL");
 		else if (syserror(tcp) || tcp->u_rval == 0)
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else if (umove(tcp, tcp->u_arg[1], &status) < 0)
 			tprintf("[?]");
 		else
@@ -710,7 +805,21 @@ int n;
 				printrusage(tcp, tcp->u_arg[3]);
 #endif /* SUNOS4 */
 			else
-				tprintf("%#x", tcp->u_arg[3]);
+				tprintf("%#lx", tcp->u_arg[3]);
+		}
+	}
+	return 0;
+}
+
+int
+internal_wait(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		if (tcp->nchildren > 0) {
+			/* There are traced children */
+			tcp->flags |= TCB_SUSPENDED;
+			tcp->waitpid = tcp->u_arg[0];
 		}
 	}
 	return 0;
@@ -924,8 +1033,14 @@ siginfo_t *sip;
 			tprintf(", si_pid=%ld, si_uid=%ld",
 				sip->si_pid, sip->si_uid);
 			switch (sip->si_code) {
-			case SI_QUEUE: case SI_TIMER:
-			case SI_ASYNCIO: case SI_MESGQ:
+			case SI_QUEUE:
+#ifdef SI_TIMER
+			case SI_TIMER:
+#endif /* SI_QUEUE */
+			case SI_ASYNCIO:
+#ifdef SI_MESGQ
+			case SI_MESGQ:
+#endif /* SI_MESGQ */
 				tprintf(", si_value=%d",
 					sip->si_value.sival_int);
 				break;
@@ -950,7 +1065,8 @@ siginfo_t *sip;
 			case SIGPOLL:
 				switch (sip->si_code) {
 				case POLL_IN: case POLL_OUT: case POLL_MSG:
-					tprintf(", si_band=%ld", sip->si_band);
+					tprintf(", si_band=%ld",
+						(long) sip->si_band);
 					break;
 				}
 				break;
@@ -962,7 +1078,7 @@ siginfo_t *sip;
 #endif
 	tprintf("}");
 }
-				
+
 int
 sys_waitid(tcp)
 struct tcb *tcp;
@@ -972,7 +1088,7 @@ struct tcb *tcp;
 
 	if (entering(tcp)) {
 		printxval(waitid_types, tcp->u_arg[0], "P_???");
-		tprintf(", %d, ", tcp->u_arg[1]);
+		tprintf(", %ld, ", tcp->u_arg[1]);
 		if (tcp->nchildren > 0) {
 			/* There are traced children */
 			tcp->flags |= TCB_SUSPENDED;
@@ -985,7 +1101,7 @@ struct tcb *tcp;
 		if (!tcp->u_arg[2])
 			tprintf("NULL");
 		else if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[2]);
+			tprintf("%#lx", tcp->u_arg[2]);
 		else if (umove(tcp, tcp->u_arg[2], &si) < 0)
 			tprintf("{???}");
 		else
@@ -1005,7 +1121,7 @@ sys_alarm(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp))
-		tprintf("%u", tcp->u_arg[0]);
+		tprintf("%lu", tcp->u_arg[0]);
 	return 0;
 }
 
@@ -1017,7 +1133,7 @@ struct tcb *tcp;
 
 	if (exiting(tcp)) {
 		if (syserror(tcp) || !verbose(tcp))
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		else if (umove(tcp, tcp->u_arg[0], &uname) < 0)
 			tprintf("{...}");
 		else if (!abbrev(tcp)) {
@@ -1028,7 +1144,9 @@ struct tcb *tcp;
 				uname.release, uname.version);
 			tprintf("machine=\"%s\"", uname.machine);
 #ifdef LINUX
+#ifndef ALPHA
 			tprintf(", domainname=\"%s\"", uname.domainname);
+#endif /* ALPHA */
 #endif /* LINUX */
 			tprintf("}");
 		}
@@ -1095,6 +1213,74 @@ static
 #endif /* !SUNOS4_KERNEL_ARCH_KLUDGE */
 struct xlat struct_user_offsets[] = {
 #ifdef LINUX
+#ifdef ALPHA
+	{ 0,		"r0"					},
+	{ 1,		"r1"					},
+	{ 2,		"r2"					},
+	{ 3,		"r3"					},
+	{ 4,		"r4"					},
+	{ 5,		"r5"					},
+	{ 6,		"r6"					},
+	{ 7,		"r7"					},
+	{ 8,		"r8"					},
+	{ 9,		"r9"					},
+	{ 10,		"r10"					},
+	{ 11,		"r11"					},
+	{ 12,		"r12"					},
+	{ 13,		"r13"					},
+	{ 14,		"r14"					},
+	{ 15,		"r15"					},
+	{ 16,		"r16"					},
+	{ 17,		"r17"					},
+	{ 18,		"r18"					},
+	{ 19,		"r19"					},
+	{ 20,		"r20"					},
+	{ 21,		"r21"					},
+	{ 22,		"r22"					},
+	{ 23,		"r23"					},
+	{ 24,		"r24"					},
+	{ 25,		"r25"					},
+	{ 26,		"r26"					},
+	{ 27,		"r27"					},
+	{ 28,		"r28"					},
+	{ 29,		"gp"					},
+	{ 30,		"fp"					},
+	{ 31,		"zero"					},
+	{ 32,		"fp0"					},
+	{ 33,		"fp"					},
+	{ 34,		"fp2"					},
+	{ 35,		"fp3"					},
+	{ 36,		"fp4"					},
+	{ 37,		"fp5"					},
+	{ 38,		"fp6"					},
+	{ 39,		"fp7"					},
+	{ 40,		"fp8"					},
+	{ 41,		"fp9"					},
+	{ 42,		"fp10"					},
+	{ 43,		"fp11"					},
+	{ 44,		"fp12"					},
+	{ 45,		"fp13"					},
+	{ 46,		"fp14"					},
+	{ 47,		"fp15"					},
+	{ 48,		"fp16"					},
+	{ 49,		"fp17"					},
+	{ 50,		"fp18"					},
+	{ 51,		"fp19"					},
+	{ 52,		"fp20"					},
+	{ 53,		"fp21"					},
+	{ 54,		"fp22"					},
+	{ 55,		"fp23"					},
+	{ 56,		"fp24"					},
+	{ 57,		"fp25"					},
+	{ 58,		"fp26"					},
+	{ 59,		"fp27"					},
+	{ 60,		"fp28"					},
+	{ 61,		"fp29"					},
+	{ 62,		"fp30"					},
+	{ 63,		"fp31"					},
+	{ 64,		"pc"					},
+#else /* !ALPHA */
+#ifdef I386
 	{ 4*EBX,		"4*EBX"					},
 	{ 4*ECX,		"4*ECX"					},
 	{ 4*EDX,		"4*EDX"					},
@@ -1112,8 +1298,37 @@ struct xlat struct_user_offsets[] = {
 	{ 4*EFL,		"4*EFL"					},
 	{ 4*UESP,		"4*UESP"				},
 	{ 4*SS,			"4*SS"					},
+#else /* !I386 */
+#ifdef M68K
+	{ 4*PT_D1,		"4*PT_D1"				},
+	{ 4*PT_D2,		"4*PT_D2"				},
+	{ 4*PT_D3,		"4*PT_D3"				},
+	{ 4*PT_D4,		"4*PT_D4"				},
+	{ 4*PT_D5,		"4*PT_D5"				},
+	{ 4*PT_D6,		"4*PT_D6"				},
+	{ 4*PT_D7,		"4*PT_D7"				},
+	{ 4*PT_A0,		"4*PT_A0"				},
+	{ 4*PT_A1,		"4*PT_A1"				},
+	{ 4*PT_A2,		"4*PT_A2"				},
+	{ 4*PT_A3,		"4*PT_A3"				},
+	{ 4*PT_A4,		"4*PT_A4"				},
+	{ 4*PT_A5,		"4*PT_A5"				},
+	{ 4*PT_A6,		"4*PT_A6"				},
+	{ 4*PT_D0,		"4*PT_D0"				},
+	{ 4*PT_USP,		"4*PT_USP"				},
+	{ 4*PT_ORIG_D0,		"4*PT_ORIG_D0"				},
+	{ 4*PT_SR,		"4*PT_SR"				},
+	{ 4*PT_PC,		"4*PT_PC"				},
+#endif /* M68K */
+#endif /* !I386 */
 	{ uoff(u_fpvalid),	"offsetof(struct user, u_fpvalid)"	},
+#ifdef I386
 	{ uoff(i387),		"offsetof(struct user, i387)"		},
+#else /* !I386 */
+#ifdef M68K
+	{ uoff(m68kfp),		"offsetof(struct user, m68kfp)"		},
+#endif /* M68K */
+#endif /* !I386 */
 	{ uoff(u_tsize),	"offsetof(struct user, u_tsize)"	},
 	{ uoff(u_dsize),	"offsetof(struct user, u_dsize)"	},
 	{ uoff(u_ssize),	"offsetof(struct user, u_ssize)"	},
@@ -1125,7 +1340,10 @@ struct xlat struct_user_offsets[] = {
 	{ uoff(u_fpstate),	"offsetof(struct user, u_fpstate)"	},
 	{ uoff(magic),		"offsetof(struct user, magic)"		},
 	{ uoff(u_comm),		"offsetof(struct user, u_comm)"		},
+#ifdef I386
 	{ uoff(u_debugreg),	"offsetof(struct user, u_debugreg)"	},
+#endif /* I386 */
+#endif /* !ALPHA */
 #endif /* LINUX */
 #ifdef SUNOS4
 	{ uoff(u_pcb),		"offsetof(struct user, u_pcb)"		},
@@ -1184,13 +1402,13 @@ struct tcb *tcp;
 {
 	char *cmd;
 	struct xlat *x;
-	int addr;
+	long addr;
 
 	cmd = xlookup(ptrace_cmds, tcp->u_arg[0]);
 	if (!cmd)
 		cmd = "PTRACE_???";
 	if (entering(tcp)) {
-		tprintf("%s, %u, ", cmd, tcp->u_arg[1]);
+		tprintf("%s, %lu, ", cmd, tcp->u_arg[1]);
 		addr = tcp->u_arg[2];
 		if (tcp->u_arg[0] == PTRACE_PEEKUSER
 			|| tcp->u_arg[0] == PTRACE_POKEUSER) {
@@ -1199,16 +1417,16 @@ struct tcb *tcp;
 					break;
 			}
 			if (!x->str)
-				tprintf("%#x, ", addr);
+				tprintf("%#lx, ", addr);
 			else if (x->val > addr && x != struct_user_offsets) {
 				x--;
-				tprintf("%s + %d, ", x->str, addr - x->val);
+				tprintf("%s + %ld, ", x->str, addr - x->val);
 			}
 			else
 				tprintf("%s, ", x->str);
 		}
 		else
-			tprintf("%#x, ", tcp->u_arg[2]);
+			tprintf("%#lx, ", tcp->u_arg[2]);
 #ifdef LINUX
 		switch (tcp->u_arg[0]) {
 		case PTRACE_PEEKDATA:
@@ -1222,7 +1440,7 @@ struct tcb *tcp;
 			printsignal(tcp->u_arg[3]);
 			break;
 		default:
-			tprintf("%#x", tcp->u_arg[3]);
+			tprintf("%#lx", tcp->u_arg[3]);
 			break;
 		}
 	} else {
@@ -1238,16 +1456,16 @@ struct tcb *tcp;
 #ifdef SUNOS4
 		if (tcp->u_arg[0] == PTRACE_WRITEDATA ||
 			tcp->u_arg[0] == PTRACE_WRITETEXT) {
-			tprintf("%u, ", tcp->u_arg[3]);
+			tprintf("%lu, ", tcp->u_arg[3]);
 			printstr(tcp, tcp->u_arg[4], tcp->u_arg[3]);
 		} else if (tcp->u_arg[0] != PTRACE_READDATA &&
 				tcp->u_arg[0] != PTRACE_READTEXT) {
-			tprintf("%#x", tcp->u_arg[3]);
+			tprintf("%#lx", tcp->u_arg[3]);
 		}
 	} else {
 		if (tcp->u_arg[0] == PTRACE_READDATA ||
 			tcp->u_arg[0] == PTRACE_READTEXT) {
-			tprintf("%u, ", tcp->u_arg[3]);
+			tprintf("%lu, ", tcp->u_arg[3]);
 			printstr(tcp, tcp->u_arg[4], tcp->u_arg[3]);
 		}
 	}
