@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,32 +26,36 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	system.c,v 2.20 1994/03/21 05:45:43 jrs Exp
+ *	$Id: system.c,v 2.31 1996/05/19 06:40:37 jrs Exp $
  */
 
 #include "defs.h"
 
 #ifdef LINUX
 
-#include <linux/mman.h>
 #include <linux/fs.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
 #include <linux/nfs.h>
-#include <linux/nfs.h>
-#define sys_personality xx_sys_personality
+#include <linux/nfs_mount.h>
+
+#include <linux/unistd.h>
+
+#ifdef __NR_personality
+/* Workaround for kernel namespace pollution. */
+#define sys_personality kernel_sys_personality
 #include <linux/personality.h>
 #undef sys_personality
-
+#endif /* __NR_personality */
 
 static struct xlat mount_flags[] = {
 	{ MS_RDONLY,	"MS_RDONLY"	},
 	{ MS_NOSUID,	"MS_NOSUID"	},
 	{ MS_NODEV,	"MS_NODEV"	},
 	{ MS_NOEXEC,	"MS_NOEXEC"	},
-	{ MS_NOSUID,	"MS_NOSUID"	},
 #ifdef MS_SYNCHRONOUS
-       { MS_SYNCHRONOUS,       "MS_SYNCHRONOUS"        },
+	{ MS_SYNCHRONOUS,"MS_SYNCHRONOUS"},
 #else
 	{ MS_SYNC,	"MS_SYNC"	},
 #endif
@@ -75,35 +75,45 @@ struct tcb *tcp;
 		printpath(tcp, tcp->u_arg[2]);
 		tprintf(", ");
 		printflags(mount_flags, tcp->u_arg[3]);
-		tprintf(", %#x", tcp->u_arg[4]);
+		tprintf(", %#lx", tcp->u_arg[4]);
 	}
 	return 0;
 }
 
-static struct xlat personalitylimits[] = {
-       { PER_LINUX,    "PER_LINUX"     },
-       { PER_SVR4,     "PER_SVR4"      },
-       { PER_SVR3,     "PER_SVR3"      },
-       { PER_SCOSVR3,  "PER_SCOSVR3"   },
-       { PER_WYSEV386, "PER_WYSEV386"  },
-       { PER_ISCR4,    "PER_ISCR4"     },
-       { PER_BSD,      "PER_BSD"       },
-       { PER_XENIX,    "PER_XENIX"     },
-       { STICKY_TIMEOUTS,      "STICKY_TIMEOUTS"       },
-       { WHOLE_SECONDS,        "WHOLE_SECONDS"         },
-       { 0,            NULL            },
+static struct xlat personality_options[] = {
+#ifdef PER_LINUX
+	{ PER_LINUX,	"PER_LINUX"	},
+#endif
+#ifdef PER_SVR4
+	{ PER_SVR4,	"PER_SVR4"	},
+#endif
+#ifdef PER_SVR3
+	{ PER_SVR3,	"PER_SVR3"	},
+#endif
+#ifdef PER_SCOSVR3
+	{ PER_SCOSVR3,	"PER_SCOSVR3"	},
+#endif
+#ifdef PER_WYSEV386
+	{ PER_WYSEV386,	"PER_WYSEV386"	},
+#endif
+#ifdef PER_ISCR4
+	{ PER_ISCR4,	"PER_ISCR4"	},
+#endif
+#ifdef PER_BSD
+	{ PER_BSD,	"PER_BSD"	},
+#endif
+#ifdef PER_XENIX
+	{ PER_XENIX,	"PER_XENIX"	},
+#endif
+	{ 0,		NULL		},
 };
 
 int
 sys_personality(tcp)
 struct tcb *tcp;
 {
-       if (entering(tcp)) {
-               /* Not the right way, but I'm not familiar with strace
-                  internals yet.. */
-               printxval(personalitylimits, tcp->u_arg[0] & PER_MASK, "PER_???");
-               addflags(personalitylimits, tcp->u_arg[0] & ~PER_MASK);
-       }
+	if (entering(tcp))
+		printxval(personality_options, tcp->u_arg[0], "PER_???");
 	return 0;
 }
 
@@ -243,7 +253,7 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		if (!(tcp->u_arg[2] & M_NEWTYPE) || umovestr(tcp,
 				tcp->u_arg[0],  sizeof type, type) < 0) {
-			tprintf("OLDTYPE:#%x", tcp->u_arg[0]);
+			tprintf("OLDTYPE:#%lx", tcp->u_arg[0]);
 		} else {
 			tprintf("\"%s\", ", type);
 		}
@@ -346,7 +356,7 @@ struct tcb *tcp;
 	if (entering(tcp)) {
 		printstr(tcp, tcp->u_arg[0], -1);
 		if (umove(tcp, tcp->u_arg[1], &e) < 0) {
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 			return 0;
 		}
 		tprintf("{fl:%u, anon:%u, ", e.ex_flags, e.ex_anon);
@@ -462,7 +472,7 @@ sys_fpathconf(tcp)
 struct tcb *tcp;
 {
 	if (entering(tcp)) {
-		tprintf("%u, ", tcp->u_arg[0]);
+		tprintf("%lu, ", tcp->u_arg[0]);
 		printxval(pathconflimits, tcp->u_arg[1], "_SC_???");
 	}
 	return 0;
@@ -472,9 +482,13 @@ struct tcb *tcp;
 
 #ifdef SVR4
 
+#ifdef HAVE_SYS_SYSCONFIG_H
 #include <sys/sysconfig.h>
-#include <sys/utsname.h>
+#endif /* HAVE_SYS_SYSCONFIG_H */
+
+#include <sys/mount.h>
 #include <sys/systeminfo.h>
+#include <sys/utsname.h>
 
 static struct xlat sysconfig_options[] = {
 #ifdef _CONFIG_NGROUPS
@@ -600,12 +614,212 @@ struct tcb *tcp;
 	else {
 		/* Technically some calls write values.  So what. */
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printpath(tcp, tcp->u_arg[1]);
-		tprintf(", %u", tcp->u_arg[2]);
+		tprintf(", %lu", tcp->u_arg[2]);
 	}
 	return 0;
 }
 
+#ifdef MIPS
+
+#include <sys/syssgi.h>
+
+static struct xlat syssgi_options[] = {
+	{ SGI_SYSID,		"SGI_SYSID"		},
+	{ SGI_RDUBLK,		"SGI_RDUBLK"		},
+	{ SGI_TUNE,		"SGI_TUNE"		},
+	{ SGI_IDBG,		"SGI_IDBG"		},
+	{ SGI_INVENT,		"SGI_INVENT"		},
+	{ SGI_RDNAME,		"SGI_RDNAME"		},
+	{ SGI_SETLED,		"SGI_SETLED"		},
+	{ SGI_SETNVRAM,		"SGI_SETNVRAM"		},
+	{ SGI_GETNVRAM,		"SGI_GETNVRAM"		},
+	{ SGI_QUERY_FTIMER,	"SGI_QUERY_FTIMER"	},
+	{ SGI_QUERY_CYCLECNTR,	"SGI_QUERY_CYCLECNTR"	},
+	{ SGI_PROCSZ,		"SGI_PROCSZ"		},
+	{ SGI_SIGACTION,	"SGI_SIGACTION"		},
+	{ SGI_SIGPENDING,	"SGI_SIGPENDING"	},
+	{ SGI_SIGPROCMASK,	"SGI_SIGPROCMASK"	},
+	{ SGI_SIGSUSPEND,	"SGI_SIGSUSPEND"	},
+	{ SGI_SETSID,		"SGI_SETSID"		},
+	{ SGI_SETPGID,		"SGI_SETPGID"		},
+	{ SGI_SYSCONF,		"SGI_SYSCONF"		},
+	{ SGI_WAIT4,		"SGI_WAIT4"		},
+	{ SGI_PATHCONF,		"SGI_PATHCONF"		},
+	{ SGI_READB,		"SGI_READB"		},
+	{ SGI_WRITEB,		"SGI_WRITEB"		},
+	{ SGI_SETGROUPS,	"SGI_SETGROUPS"		},
+	{ SGI_GETGROUPS,	"SGI_GETGROUPS"		},
+	{ SGI_SETTIMEOFDAY,	"SGI_SETTIMEOFDAY"	},
+	{ SGI_SETTIMETRIM,	"SGI_SETTIMETRIM"	},
+	{ SGI_GETTIMETRIM,	"SGI_GETTIMETRIM"	},
+	{ SGI_SPROFIL,		"SGI_SPROFIL"		},
+	{ SGI_RUSAGE,		"SGI_RUSAGE"		},
+	{ SGI_SIGSTACK,		"SGI_SIGSTACK"		},
+	{ SGI_SIGSTATUS,	"SGI_SIGSTATUS"		},
+	{ SGI_NETPROC,		"SGI_NETPROC"		},
+	{ SGI_SIGALTSTACK,	"SGI_SIGALTSTACK"	},
+	{ SGI_BDFLUSHCNT,	"SGI_BDFLUSHCNT"	},
+	{ SGI_SSYNC,		"SGI_SSYNC"		},
+	{ SGI_NFSCNVT,		"SGI_NFSCNVT"		},
+	{ SGI_GETPGID,		"SGI_GETPGID"		},
+	{ SGI_GETSID,		"SGI_GETSID"		},
+	{ SGI_IOPROBE,		"SGI_IOPROBE"		},
+	{ SGI_CONFIG,		"SGI_CONFIG"		},
+	{ SGI_ELFMAP,		"SGI_ELFMAP"		},
+	{ SGI_MCONFIG,		"SGI_MCONFIG"		},
+	{ SGI_GETPLABEL,	"SGI_GETPLABEL"		},
+	{ SGI_SETPLABEL,	"SGI_SETPLABEL"		},
+	{ SGI_GETLABEL,		"SGI_GETLABEL"		},
+	{ SGI_SETLABEL,		"SGI_SETLABEL"		},
+	{ SGI_SATREAD,		"SGI_SATREAD"		},
+	{ SGI_SATWRITE,		"SGI_SATWRITE"		},
+	{ SGI_SATCTL,		"SGI_SATCTL"		},
+	{ SGI_LOADATTR,		"SGI_LOADATTR"		},
+	{ SGI_UNLOADATTR,	"SGI_UNLOADATTR"	},
+#ifdef SGI_RECVLMSG
+	{ SGI_RECVLMSG,		"SGI_RECVLMSG"		},
+#endif
+	{ SGI_PLANGMOUNT,	"SGI_PLANGMOUNT"	},
+	{ SGI_GETPSOACL,	"SGI_GETPSOACL"		},
+	{ SGI_SETPSOACL,	"SGI_SETPSOACL"		},
+	{ SGI_RMI_FIXECC,	"SGI_RMI_FIXECC"	},
+	{ SGI_R4K_CERRS,	"SGI_R4K_CERRS"		},
+	{ SGI_GET_EVCONF,	"SGI_GET_EVCONF"	},
+	{ SGI_MPCWAROFF,	"SGI_MPCWAROFF"		},
+	{ SGI_SET_AUTOPWRON,	"SGI_SET_AUTOPWRON"	},
+	{ SGI_SPIPE,		"SGI_SPIPE"		},
+	{ SGI_SYMTAB,		"SGI_SYMTAB"		},
+#ifdef SGI_SET_FPDEBUG
+	{ SGI_SET_FPDEBUG,	"SGI_SET_FPDEBUG"	},
+#endif
+	{ SGI_TOSSTSAVE,	"SGI_TOSSTSAVE"		},
+	{ SGI_FDHI,		"SGI_FDHI"		},
+	{ SGI_MINRSS,		"SGI_MINRSS"		},
+	{ 0,			NULL			},
+};
+
+int
+sys_syssgi(tcp)
+struct tcb *tcp;
+{
+	int i;
+
+	if (entering(tcp)) {
+		printxval(syssgi_options, tcp->u_arg[0], "SGI_???");
+		switch (tcp->u_arg[0]) {
+		default:
+			for (i = 1; i < tcp->u_nargs; i++)
+				tprintf(", %#lx", tcp->u_arg[i]);
+			break;
+		}
+	}
+	return 0;
+}
+
+#include <sys/types.h>
+#include <rpc/rpc.h>
+struct cred;
+struct uio;
+#include <sys/fsid.h>
+#include <sys/vnode.h>
+#include <sys/fs/nfs.h>
+#include <sys/fs/nfs_clnt.h>
+
+static struct xlat mount_flags[] = {
+	{ MS_RDONLY,	"MS_RDONLY"	},
+	{ MS_FSS,	"MS_FSS"	},
+	{ MS_DATA,	"MS_DATA"	},
+	{ MS_NOSUID,	"MS_NOSUID"	},
+	{ MS_REMOUNT,	"MS_REMOUNT"	},
+	{ MS_NOTRUNC,	"MS_NOTRUNC"	},
+	{ MS_GRPID,	"MS_GRPID"	},
+	{ MS_NODEV,	"MS_NODEV"	},
+	{ MS_BEFORE,	"MS_BEFORE"	},
+	{ MS_AFTER,	"MS_AFTER"	},
+	{ 0,		NULL		},
+};
+
+static struct xlat nfs_flags[] = {
+	{ NFSMNT_SOFT,		"NFSMNT_SOFT"		},
+	{ NFSMNT_WSIZE,		"NFSMNT_WSIZE"		},
+	{ NFSMNT_RSIZE,		"NFSMNT_RSIZE"		},
+	{ NFSMNT_TIMEO,		"NFSMNT_TIMEO"		},
+	{ NFSMNT_RETRANS,	"NFSMNT_RETRANS"	},
+	{ NFSMNT_HOSTNAME,	"NFSMNT_HOSTNAME"	},
+	{ NFSMNT_INT,		"NFSMNT_INT"		},
+	{ NFSMNT_NOAC,		"NFSMNT_NOAC"		},
+	{ NFSMNT_ACREGMIN,	"NFSMNT_ACREGMIN"	},
+	{ NFSMNT_ACREGMAX,	"NFSMNT_ACREGMAX"	},
+	{ NFSMNT_ACDIRMIN,	"NFSMNT_ACDIRMIN"	},
+	{ NFSMNT_ACDIRMAX,	"NFSMNT_ACDIRMAX"	},
+	{ NFSMNT_PRIVATE,	"NFSMNT_PRIVATE"	},
+	{ NFSMNT_SYMTTL,	"NFSMNT_SYMTTL"		},
+	{ NFSMNT_LOOPBACK,	"NFSMNT_LOOPBACK"	},
+	{ NFSMNT_BASETYPE,	"NFSMNT_BASETYPE"	},
+	{ NFSMNT_NAMEMAX,	"NFSMNT_NAMEMAX"	},
+	{ 0,			NULL			},
+};
+
+int
+sys_mount(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		printpath(tcp, tcp->u_arg[1]);
+		tprintf(", ");
+		printflags(mount_flags, tcp->u_arg[2]);
+		if (tcp->u_arg[2] & (MS_FSS | MS_DATA)) {
+			tprintf(", ");
+			tprintf("%ld", tcp->u_arg[3]);
+		}
+		if (tcp->u_arg[2] & MS_DATA) {
+			int nfs_type = sysfs(GETFSIND, FSID_NFS);
+
+			tprintf(", ");
+			if (tcp->u_arg[3] == nfs_type) {
+				struct nfs_args args;
+				if (umove(tcp, tcp->u_arg[4], &args) < 0)
+					tprintf("%#lx", tcp->u_arg[4]);
+				else {
+					tprintf("addr=");
+					printsock(tcp, (int) args.addr);
+					tprintf(", flags=");
+					if (!printflags(nfs_flags, args.flags))
+						tprintf("NFSMNT_???");
+					tprintf(", hostname=");
+					printstr(tcp, (int) args.hostname, -1);
+					tprintf(", ...}");
+				}
+			}
+			else
+				tprintf("%#lx", tcp->u_arg[4]);
+			tprintf(", %ld", tcp->u_arg[5]);
+		}
+	}
+	return 0;
+}
+
+#else /* !MIPS */
+
+int
+sys_mount(tcp)
+struct tcb *tcp;
+{
+	if (entering(tcp)) {
+		printpath(tcp, tcp->u_arg[0]);
+		tprintf(", ");
+		printpath(tcp, tcp->u_arg[1]);
+		tprintf(", ...");
+	}
+	return 0;
+}
+
+#endif /* !MIPS */
+
+
 #endif /* SVR4 */
