@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 1991, 1992 Paul Kranenburg <pk@cs.few.eur.nl>
  * Copyright (c) 1993 Branko Lankester <branko@hacktic.nl>
- * Copyright (c) 1993, 1994 Rick Sladkey <jrs@world.std.com>
+ * Copyright (c) 1993, 1994, 1995, 1996 Rick Sladkey <jrs@world.std.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -12,11 +12,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Paul Kranenburg,
- *      Branko Lankester and Rick Sladkey.
- * 4. The name of the author may not be used to endorse or promote products
+ * 3. The name of the author may not be used to endorse or promote products
  *    derived from this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
@@ -30,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	time.c,v 2.23 1994/06/29 04:43:27 jrs Exp
+ *	$Id: time.c,v 2.28 1996/05/17 02:38:40 jrs Exp $
  */
 
 #include "defs.h"
@@ -43,18 +39,18 @@
 void
 printtv(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	struct timeval tv;
 
 	if (addr == 0)
 		tprintf("NULL");
 	else if (!verbose(tcp))
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 	else if (umove(tcp, addr, &tv) < 0)
 		tprintf("{...}");
 	else
-		tprintf("{%lu, %lu}", tv.tv_sec, tv.tv_usec);
+		tprintf("{%lu, %lu}", (long) tv.tv_sec, (long) tv.tv_usec);
 }
 
 int
@@ -63,7 +59,7 @@ struct tcb *tcp;
 {
 	if (exiting(tcp)) {
 #ifndef SVR4
-		printnum(tcp, tcp->u_arg[0], "%d");
+		printnum(tcp, tcp->u_arg[0], "%ld");
 #endif /* SVR4 */
 	}
 	return 0;
@@ -74,7 +70,7 @@ sys_stime(tcp)
 struct tcb *tcp;
 {
 	if (exiting(tcp)) {
-		printnum(tcp, tcp->u_arg[0], "%d");
+		printnum(tcp, tcp->u_arg[0], "%ld");
 	}
 	return 0;
 }
@@ -85,7 +81,7 @@ struct tcb *tcp;
 {
 	if (exiting(tcp)) {
 		if (syserror(tcp)) {
-			tprintf("%#x, %#x",
+			tprintf("%#lx, %#lx",
 				tcp->u_arg[0], tcp->u_arg[1]);
 			return 0;
 		}
@@ -121,7 +117,7 @@ struct tcb *tcp;
 		tprintf(", ");
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printtv(tcp, tcp->u_arg[1]);
 	}
@@ -138,20 +134,20 @@ static struct xlat which[] = {
 static void
 printitv(tcp, addr)
 struct tcb *tcp;
-int addr;
+long addr;
 {
 	struct itimerval itv;
 
 	if (addr == 0)
 		tprintf("NULL");
 	else if (!verbose(tcp))
-		tprintf("%#x", addr);
+		tprintf("%#lx", addr);
 	else if (umove(tcp, addr, &itv) < 0)
 		tprintf("{...}");
 	else {
 		tprintf("{it_interval={%lu, %lu}, it_value={%lu, %lu}}",
-		itv.it_interval.tv_sec, itv.it_interval.tv_usec,
-		itv.it_value.tv_sec, itv.it_value.tv_usec);
+		(long) itv.it_interval.tv_sec, (long) itv.it_interval.tv_usec,
+		(long) itv.it_value.tv_sec, (long) itv.it_value.tv_usec);
 	}
 }
 
@@ -164,7 +160,7 @@ struct tcb *tcp;
 		tprintf(", ");
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[1]);
+			tprintf("%#lx", tcp->u_arg[1]);
 		else
 			printitv(tcp, tcp->u_arg[1]);
 	}
@@ -182,7 +178,7 @@ struct tcb *tcp;
 		tprintf(", ");
 	} else {
 		if (syserror(tcp))
-			tprintf("%#x", tcp->u_arg[2]);
+			tprintf("%#lx", tcp->u_arg[2]);
 		else
 			printitv(tcp, tcp->u_arg[2]);
 	}
@@ -201,30 +197,32 @@ struct tcb *tcp;
 		if (tcp->u_arg[0] == 0)
 			tprintf("NULL");
 		else if (syserror(tcp) || !verbose(tcp))
-			tprintf("%#x", tcp->u_arg[0]);
+			tprintf("%#lx", tcp->u_arg[0]);
 		else if (umove(tcp, tcp->u_arg[0], &txc) < 0)
 			tprintf("{...}");
 		else {
-#if (LINUX_VERSION_CODE <= 66061)
-			tprintf("{modes=%d, offset=%ld, frequency=%ld, ",
+#if LINUX_VERSION_CODE < 66332
+			tprintf("{mode=%d, offset=%ld, frequency=%ld, ",
 				txc.mode, txc.offset, txc.frequency);
 			tprintf("maxerror=%ld, esterror=%lu, status=%u, ",
 				txc.maxerror, txc.esterror, txc.status);
-			tprintf("pll time_constant=%ld, precision=%lu, ",
+			tprintf("time_constant=%ld, precision=%lu, ",
 				txc.time_constant, txc.precision);
 			tprintf("tolerance=%ld, time={%lu, %lu}}",
-				txc.tolerance,
-				txc.time.tv_sec, txc.time.tv_usec);
+				txc.tolerance, (long) txc.time.tv_sec,
+				(long) txc.time.tv_usec);
 #else
-			tprintf("{modes=%d, offset=%ld, frequency=%ld, ",
+			tprintf("{modes=%d, offset=%ld, freq=%ld, ",
 				txc.modes, txc.offset, txc.freq);
 			tprintf("maxerror=%ld, esterror=%lu, status=%u, ",
 				txc.maxerror, txc.esterror, txc.status);
-			tprintf("pll time_constant=%ld, precision=%lu, ",
+			tprintf("constant=%ld, precision=%lu, ",
 				txc.constant, txc.precision);
 			tprintf("tolerance=%ld, time={%lu, %lu}}",
-				txc.tolerance,
-				txc.time.tv_sec, txc.time.tv_usec);
+				txc.tolerance, (long) txc.time.tv_sec,
+				(long) txc.time.tv_usec);
+			/* there's a bunch of other stuff, but it's not
+			 * worth the time or the trouble to include */
 #endif
 		}
 	}
