@@ -26,7 +26,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *	$Id: util.c,v 2.37 1996/05/21 03:32:23 jrs Exp $
+ *	$Id: util.c,v 2.38 1996/10/28 03:30:42 jrs Exp $
  */
 
 #include "defs.h"
@@ -604,9 +604,9 @@ long addr;
 int len;
 char *laddr;
 {
-#ifndef SUNOS4
+#ifdef SRVR4
 	return umoven(tcp, addr, len, laddr);
-#else /* SUNOS4 */
+#else /* !SVR4 */
 	int pid = tcp->pid;
 	int i, n, m;
 	union {
@@ -645,7 +645,7 @@ char *laddr;
 		addr += sizeof(long), laddr += m, len -= m;
 	}
 	return 0;
-#endif /* SUNOS4 */
+#endif /* !SVR4 */
 }
 
 #ifdef LINUX
